const mongoose = require('../../common/services/mongoose.service').mongoose;
const Schema = mongoose.Schema;
const {queryFormatter,queryBuilder_string,
    queryBuilder_number,
    queryBuilder_date,
    queryBuilder_array,
    queryBuilder_range_array} = require("../../common/functions/queryutilMongo")
const endpointsSchema = new Schema({
    	createBy : { type: String,required:true,default:''},
			createAt : { type: Date,required:true},
			updateBy : { type: String},
			updateAt : { type: Date},
			name : { type: String,required:true,default:'',maxLength:50,minLength:2},
			path : { type: String,required:true,default:'',maxLength:100,minLength:1},
			method : { type: String},
			project : { type: Schema.Types.ObjectId, ref: "Project" ,required:true},
			headers : [{ type: Schema.Types.ObjectId, ref: "Items"}],
            success : { type: String},
            validationType : { type: String}
});

endpointsSchema.virtual('id').get(function () {
    return this._id.toHexString();
});

// Ensure virtual fields are serialised.
endpointsSchema.set('toJSON', {
    virtuals: true
});

endpointsSchema.findById = function (cb) {
    return this.model('Endpoints').find({id: this.id}, cb);
};

const Endpoints = mongoose.model('Endpoints', endpointsSchema);


exports.findById = (id,extraField) => {
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:id}
    return Endpoints.findOne(queries)
        .populate({ path: 'project', select: 'id projectName' })
        .populate({ path: 'headers', select: 'id itemName value' })
        .then((result) => {
            result = result.toJSON();
            delete result._id;
            delete result.__v;
            return result;
        });
};

exports.createEndpoints = (endpointsData) => {
    return new Promise(async(resolve, reject) => {

        let pathExist= await this.listAll({path:endpointsData.path,method:endpointsData.method})
     
       if (pathExist && pathExist.length>0) {
        return reject("Path  already exists");
        
    }
    
    const endpoints = new Endpoints(endpointsData);
    endpoints.save(function (err, saved) {
        if (err) {
            return reject(err);
        }
        resolve(saved)
    });
    });
};

exports.list = (perPage, page , query ) => {
        var _query={};
        let sortBy='_id'
        let sortDirection=-1
        
    if(query.createBy){
        let createBy_= queryBuilder_string(query,'createBy');
        _query={..._query,...createBy_}
    }


        if(query.createAt){
            let createAt_= queryBuilder_date(query,'createAt');
            _query={..._query,...createAt_}
        }
        if(query.createAt_array){
            let createAt_a= queryBuilder_range_array(query,'createAt',"date");
            _query={..._query,...createAt_a}
        }
    
    

    if(query.updateBy){
        let updateBy_= queryBuilder_string(query,'updateBy');
        _query={..._query,...updateBy_}
    }


        if(query.updateAt){
            let updateAt_= queryBuilder_date(query,'updateAt');
            _query={..._query,...updateAt_}
        }
        if(query.updateAt_array){
            let updateAt_a= queryBuilder_range_array(query,'updateAt',"date");
            _query={..._query,...updateAt_a}
        }
    
    

    if(query.name){
        let name_= queryBuilder_string(query,'name');
        _query={..._query,...name_}
    }

     
    

    if(query.path){
        let path_= queryBuilder_string(query,'path');
        _query={..._query,...path_}
    }


    if(query.project){
        let project_= queryBuilder_string(query,'project');
        _query={..._query,...project_}
    }


    if(query.headers){
        if(Array.isArray(query.headers)){
            const regexheaders = query.headers.map(value => new RegExp(value, 'i'));
            _query['headers'] = { $in: regexheaders } 
        }else{
            _query['headers'] = { $in: [new RegExp(query.headers, "i")] } 
        }
    }
        
        if(query.sortBy){
            sortBy = query.sortBy;
        }
        if(query.sortDirection){
            sortDirection = query.sortDirection;
        }
        var sortBoj={[sortBy]:sortDirection};
        return new Promise((resolve, reject) => {
        Endpoints.find(_query)
            .populate({ path: 'project', select: 'id projectName' })
            .populate({ path: 'headers', select: 'id itemName value' })
            .limit(perPage)
            .sort(sortBoj)
            .skip(perPage * page)
            .exec(function (err, endpoints) {
                if (err) {
                    reject(err);
                } else {
                    Endpoints.countDocuments(_query).exec().then((total)=>{
                        const promises = { docs: endpoints , count: total ,perpage:perPage,page:page };
                        resolve(promises);
                    }).catch((err2)=>{
                        reject(err2);
                    })
                }
            })
    });
};
exports.listAll = ( query={} ) => {
    var _query={...query};
    let sortBy='_id'
    let sortDirection=-1
    let max_limit = 300;
    if(query.sortBy){
        sortBy = query.sortBy;
    }
    if(query.sortDirection){
        sortDirection = query.sortDirection;
    }
    var sortBoj={[sortBy]:sortDirection};
    console.log(_query)
    return new Promise((resolve, reject) => {
    Endpoints.find(_query)
        .populate({ path: 'project', select: 'id projectName' })
        .populate({ path: 'headers', select: 'id itemName value' })
        .limit(max_limit) 
        .sort(sortBoj)
        .exec(function (err, endpoints) {
                if (err) {
                    reject(err);
                } else {
                resolve(endpoints);
                }
            })
    });
};
exports.listAllPublish = ( query={} ) => {
    var _query={...query};
    let sortBy='_id'
    let sortDirection=-1
    let max_limit = 300;
    if(query.sortBy){
        sortBy = query.sortBy;
    }
    if(query.sortDirection){
        sortDirection = query.sortDirection;
    }
    var sortBoj={[sortBy]:sortDirection};
    console.log(_query)
    return new Promise((resolve, reject) => {
    Endpoints.find(_query)
        .select('name path headers project method success validationType')
        .populate({ path: 'headers', select: 'id itemName value' })
        .limit(max_limit) 
        .sort(sortBoj)
        .exec(function (err, endpoints) {
                if (err) {
                    reject(err);
                } else {
                resolve(endpoints);
                }
            })
    });
};

exports.listSuggestions = (query ) => {
    var { search, ...queryWithoutSearch } = query
    var _query=queryWithoutSearch
    if(search){
        _query['$or'] =[ 
            {"name": {$regex: search, $options: "i"}},
				{"path": {$regex: search, $options: "i"}},
				{"project": {$regex: search, $options: "i"}},
				{"headers": {$in: [search]}}
        ]
    }
    return new Promise((resolve, reject) => {
        Endpoints.find(_query)
            .exec(function (err, endpoints) {
                if (err) {
                    reject(err);
                } else {
                    resolve(endpoints);
                }
            })
    });
};
exports.patchEndpoints = (id, endpointsData,extraField={}) => {
    
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:id}
    return new Promise((resolve, reject) => {
        
        Endpoints.findOne(queries, function (err, endpoints) {
            if (err) reject(err);
            for (let i in endpointsData) {
                endpoints[i] = endpointsData[i];
            }
            endpoints.save(function (err, updatedEndpoints) {
                if (err) return reject(err);
                resolve(updatedEndpoints);
            });
        });
    })

};

exports.removeById = (endpointsId,extraField={}) => {
    var extraQuery =queryFormatter(extraField);
    var queries = {...extraQuery,_id:endpointsId}
    return new Promise((resolve, reject) => {
        Endpoints.findOneAndDelete(queries, (err) => {
            if (err) {
                reject(err);
            } else {
                resolve(err);
            }
        });
    });
};



    